@@ -8,20 +8,25 @@ pdflatex latex-via-exemplos-poster
 pdflatex latex-via-exemplos-slides
 pdflatex latex-via-exemplos-slides
 pdflatex latex-via-exemplos-certificado
+pdflatex latex-via-exemplos-slides
+pdflatex latex-via-exemplos-cracha
+pdflatex latex-via-exemplos-folder
+pdflatex latex-via-exemplos-todonotes
+pdflatex latex-via-exemplos-todonotes
     
-# compile documento (two sides)
+# compile documento (one sided)
 xelatex latex-via-exemplos
 bibtex latex-via-exemplos
 makeindex latex-via-exemplos
 xelatex latex-via-exemplos
 xelatex latex-via-exemplos
 
-# compile document (one sides)
-xelatex latex-via-exemplos-oneside
-bibtex latex-via-exemplos-oneside
-makeindex latex-via-exemplos-oneside
-xelatex latex-via-exemplos-oneside
-xelatex latex-via-exemplos-oneside
+# compile document (two sided)
+xelatex latex-via-exemplos-twosides
+bibtex latex-via-exemplos-twosides
+makeindex latex-via-exemplos-twosides
+xelatex latex-via-exemplos-twosides
+xelatex latex-via-exemplos-twosides
 
 # clean up
 rm *.aux
@@ -41,16 +46,25 @@ rm *.nav
 rm *.snm
 rm *.vrb
 
+rm *.tdo
+
+# rm *.csv
+
 # remove automatic created auxiliar tex and txt files
 rm latex-via-exemplos-tabela.txt
 rm latex-via-exemplos-ans.tex
-rm latex-via-exemplos-oneside-ans.tex
+rm latex-via-exemplos-twosides-ans.tex
+
+# rm latex-via-exemplos-*-exercise-body.tex
 
 # remove auxiliar pdf files
-rm latex-via-exemplos-certificado.pdf
 rm latex-via-exemplos-fig.pdf
 rm latex-via-exemplos-poster.pdf
 rm latex-via-exemplos-slides.pdf
+rm latex-via-exemplos-certificado.pdf
+rm latex-via-exemplos-cracha.pdf
+rm latex-via-exemplos-folder.pdf
+rm latex-via-exemplos-todonotes.pdf
 
 #end
 
