@@ -8,7 +8,7 @@ for ($d=0; $d<=1; $d++){
 		$font_name="otf-$base_name-$dir";
 		open(OPL, ">opl/$font_name.opl") || die "cannot make file";
 		print OPL "(OFMLEVEL D 1)\n";
-		print OPL "(FONTDIR TL)\n";
+		printf OPL "(FONTDIR %s)\n", $d==1 ? "RT" : "TL";
 		print OPL "(FAMILY HIRAMIN)\n";
 		print OPL "(FACE F MRR)\n";
 		print OPL "(CODINGSCHEME UNSPECIFIED)\n";
@@ -25,7 +25,7 @@ for ($d=0; $d<=1; $d++){
 		printf OPL "   (XHEIGHT R %f)\n", $font_at;
 		printf OPL "   (QUAD R %f)\n", $font_at;
 		print OPL "   )\n";
-		for($cidcode=0; $cidcode<=23057; $cidcode++){
+		for($cidcode=0; $cidcode<=23059; $cidcode++){
 			if ($dir eq "h"){
 				if (($cidcode >= 231) && ($cidcode <= 632)){
 					$width=0.5;
